//
//
// hello.array.44.c
//
// Charlieplex LED array hello-world
//
// Neil Gershenfeld
// 11/13/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port_1 PORTA
#define led_direction_1 DDRA
#define led_port_2 PORTB
#define led_direction_2 DDRB

#define A (1 << PB0) // row 1
#define B (1 << PB1) // row 2
#define C (1 << PA3) // row 3
#define D (1 << PA4) // row 4
#define E (1 << PA5) // row 5

#define sensor_port PORTA
#define ref (0 << REFS1) | (0 << REFS0) // Vcc reference voltage
#define sensor_1 (0 << MUX2) | (0 << MUX1) | (0 << MUX0) // PA0
#define sensor_2 (0 << MUX2) | (0 << MUX1) | (1 << MUX0) // PA1
#define sensor_3 (0 << MUX2) | (1 << MUX1) | (0 << MUX0) // PA2



int read_ad_pin(uint8_t pin) {
	
	uint8_t hi_value, lo_value;
	int value;
	
	// set the A/D pin
	ADMUX = ref | pin;
	
	// initiate conversion
	ADCSRA |= (1 << ADSC);
	
	// wait for completion
	while (ADCSRA & (1 << ADSC))
		;
	
	// save result
	lo_value = ADCL;
	hi_value = ADCH;
	
	// process result
	value = 256 * hi_value + lo_value;
	
	return value;
	
}

void flash(uint8_t from, uint8_t to, uint8_t delay) {
    //
    // source from, sink to, flash
    //
    static uint8_t i;
    set(led_port_1,from);
    clear(led_port_1,to);
    set(led_port_2,from);
    clear(led_port_2,to);
    output(led_direction_1,from);
    output(led_direction_1,to);
    output(led_direction_2,from);
    output(led_direction_2,to);
    for (i = 0; i < delay; ++i)
        led_delay();
    input(led_direction_1,from);
    input(led_direction_1,to);
    input(led_direction_2,from);
    input(led_direction_2,to);
}

void led_cycle(uint8_t number, uint8_t delay) {
    //
    // cycle through LEDs
    //
    uint8_t i;
    for (i = 0; i < number; ++i) {
        flash(B,A,delay);
        flash(C,A,delay);
        flash(D,A,delay);
        flash(E,A,delay);
        flash(A,B,delay);
        flash(C,B,delay);
        flash(D,B,delay);
        flash(E,B,delay);
        flash(A,C,delay);
        flash(B,C,delay);
        flash(D,C,delay);
        flash(E,C,delay);
        flash(A,D,delay);
        flash(B,D,delay);
        flash(C,D,delay);
        flash(E,D,delay);
        flash(A,E,delay);
        flash(B,E,delay);
        flash(C,E,delay);
        flash(D,E,delay);
    }
}

void led_cycle_rightArrow(uint8_t number, uint8_t delay) {
    uint8_t i;
    for (i = 0; i < number; ++i) {
        flash(E,C,delay);
        flash(D,B,delay);
        flash(C,D,delay);
        flash(C,A,delay);
        flash(B,E,delay);
        
        flash(B,A,delay);
        flash(C,B,delay);
        flash(D,C,delay);
        flash(B,D,delay);
        flash(A,E,delay);
    }
}

void led_cycle_leftArrow(uint8_t number, uint8_t delay) {
    uint8_t i;
    for (i = 0; i < number; ++i) {
        flash(A,C,delay);
        flash(C,B,delay);
        flash(D,A,delay);
        flash(B,D,delay);
        flash(C,E,delay);
        
        flash(E,A,delay);
        flash(D,B,delay);
        flash(B,C,delay);
        flash(C,D,delay);
         flash(D,E,delay);
    }
}


int main(void) {
    static int sensor_1_value, sensor_2_value, sensor_3_value;
    //bool right_steer, left_steer,brake,normal;
    int current_sensor_2_value,last_sensor_2_value, steer_speed;
    steer_speed=0;
    
    // set clock divider to /1
    CLKPR = (1 << CLKPCE);
    CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
    
    // init A/D
	ADMUX = ref; // Vcc ref
	ADCSRA = (1 << ADEN) // enable
    | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
	ADCSRB = (0 << ADLAR); // right adjust
    
    last_sensor_2_value=read_ad_pin(sensor_2);
    
    // main loop
    //
    while (1) {
        
        sensor_1_value=read_ad_pin(sensor_1); // slow the bike
        current_sensor_2_value=read_ad_pin(sensor_2);
        
        if(current_sensor_2_value!=last_sensor_2_value){
            steer_speed=current_sensor_2_value-last_sensor_2_value;
            //last_sensor_2_value=current_sensor_2_value;
        }
        
        if (steer_speed>8) {
            led_cycle_leftArrow(1,1);
        } else if(steer_speed<-8){
            led_cycle_rightArrow(1,1);
        } else if(sensor_1_value<1000 && steer_speed>-8 && steer_speed<8){
            led_cycle(1,1);
        }
        else{
        }
    }

}
